package cz.cuni.mff.nprg013.tests.robots;


import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import cz.cuni.mff.nprg013.tests.robots.model.Maze;
import cz.cuni.mff.nprg013.tests.robots.model.MazeFields;
import cz.cuni.mff.nprg013.tests.robots.model.RobotModel;
import cz.cuni.mff.nprg013.tests.robots.model.RobotModels;
import cz.cuni.mff.nprg013.tests.robots.model.RobotOrientation;
import cz.cuni.mff.nprg013.tests.robots.model.RobotState;

@Path("/robots/{robot_id}")
public class Robot {
    private final Gson gson;
    
    public static class RobotDetails {
        public final String id;
        public final String name;
        
        public final RobotState state;
        
        public final RobotOrientation direction;
        public final int posX;
        public final int posY;
        
        public final String surrounding;
    
        public RobotDetails(RobotModel robot, Maze m) {
            id = robot.getId();
            name = robot.getName();
            
            state = robot.getState();
            direction = robot.getDirection();
            
            posX = robot.getPosX();
            posY = robot.getPosY();
            
            StringBuilder sb = new StringBuilder();
            for(int iy = posY - 2; iy <= posY + 2; iy++) {
                for(int ix = posX - 2; ix <= posX + 2; ix++) {
                    if (ix >= 0 && ix < Maze.WIDTH && iy >= 0 && iy < Maze.HEIGHT) {
                        MazeFields mf = m.getMaze(iy, ix);
                        sb.append(mf.mazeMark);
                    } else {
                        sb.append(MazeFields.WALL.mazeMark);
                    }
                }
                sb.append('\n');
            }
            surrounding = sb.toString();
        }

        public String getId() {
            return id;
        }

        public String getName() {
            return name;
        }

        public RobotState getState() {
            return state;
        }

        public RobotOrientation getDirection() {
            return direction;
        }

        public int getPosX() {
            return posX;
        }

        public int getPosY() {
            return posY;
        }

        public String getSurrounding() {
            return surrounding;
        }

    }
    
    
    public Robot() {
        super();
        GsonBuilder gb = new GsonBuilder();
        gb.setPrettyPrinting();
        this.gson = gb.create();
    }


    
    @GET
    public String getRobot(@PathParam("robot_id") String id ) {
        RobotModels rm = RobotModels.getInstance();
        RobotDetails robotDetails = null;
        synchronized(rm) {
            RobotModel robot = rm.getRobotById(id);
            if (robot == null) {
                return "{ \"error\": \"Invalid input. Robot with given ID does not exists\", \"robot_id\": \"" + id + "\"}";
            }
            Maze m = new Maze();
            m = m.addRobots( rm.getRobots());
            System.out.println(m);
            robotDetails = new RobotDetails(robot, m);
        }
        return gson.toJson(robotDetails);
    }

}
