package cz.cuni.mff.nprg013.tests.robots;


import java.util.Arrays;

import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonSyntaxException;

import cz.cuni.mff.nprg013.tests.robots.model.Actions;
import cz.cuni.mff.nprg013.tests.robots.model.RobotModel;
import cz.cuni.mff.nprg013.tests.robots.model.RobotModels;

@Path("/robots/{robot_id}/actions")
public class RobotActions {
    private final Gson gson;
    
    public RobotActions() {
        super();
        GsonBuilder gb = new GsonBuilder();
        gb.setPrettyPrinting();
        this.gson = gb.create();
    }

    public static class RobotActionsDetails {
        public final String id;
        public final String name;
        
        public final Actions[] executedActions;
        public final Actions[] plannedActions;
        public RobotActionsDetails(RobotModel robot) {
            super();
            this.id = robot.getId();
            this.name = robot.getName();
            this.executedActions = robot.getExecutedActions();
            this.plannedActions = robot.getPlannedActions();
        }
        
        public String getId() {
            return id;
        }
        public String getName() {
            return name;
        }
        public Actions[] getExecutedActions() {
            return executedActions;
        }
        public Actions[] getPlannedActions() {
            return plannedActions;
        }
    }

    public static class PostAction {
        public Actions action;

        public Actions getAction() {
            return action;
        }

        public void setAction(Actions action) {
            this.action = action;
        }
        
    }
    @GET
    public String getRobotActions(@PathParam("robot_id") String id ) {
        RobotModels rm = RobotModels.getInstance();
        RobotActionsDetails details = null;
        synchronized(rm) {
            RobotModel robot = rm.getRobotById(id);
            if (robot == null) {
                return "{ \"error\": \"Invalid input. Robot with given ID does not exists\", \"robot_id\": \"" + id + "\"}";
            }
            details = new RobotActionsDetails(robot);
        }
        return gson.toJson(details);
    }

    
    @POST
    public String addRobotAction(@PathParam("robot_id") String id, String input) {
        try {
            PostAction request = gson.fromJson(input, PostAction.class);
            RobotModels rm = RobotModels.getInstance();
            
            synchronized(rm) {
                RobotModel robot = rm.getRobotById(id);
                if (robot == null) {
                    return "{ \"error\": \"Invalid input. Robot with given ID does not exists\", \"robot_id\": \"" + id + "\"}";
                }
                if (request.action == null) {
                    return "{ \"error\": \"Invalid input. Action cannot be null or wrong value passed. Permitted values are " + Arrays.toString(Actions.values()) + "\", \"robot_id\": \"" + id + "\"}";
                }
                robot.addAction(request.action);
            }
            return getRobotActions(id);
        } catch (JsonSyntaxException e) {
            return "{ \"error\": \"Invalid input. Not a valid json\", \"exception\": \"" + e.toString() + "\"}";
        }
        
    }
}
