package cz.cuni.mff.nprg013.tests.robots;

import java.util.ArrayList;
import java.util.List;

import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonSyntaxException;

import cz.cuni.mff.nprg013.tests.robots.model.RobotModel;
import cz.cuni.mff.nprg013.tests.robots.model.RobotModels;

@Path("/robots")
public class Robots {
    private final Gson gson;
    
    public static class RobotIdName {
        public final String id;
        public final String name;
        
        public RobotIdName(RobotModel robot) {
            id = robot.getId();
            name = robot.getName();
        }

        public String getId() {
            return id;
        }

        public String getName() {
            return name;
        }
    }
    
   public static class RobotName {
       public String name;
       
       public void setName(String name) {
           this.name = name;
       }

       public String getName() {
          return name;
       }
       
   }
    
    public Robots() {
        super();
        GsonBuilder gb = new GsonBuilder();
        gb.setPrettyPrinting();
        this.gson = gb.create();
    }


    @GET
    public String getRobots() {
        RobotModels rm = RobotModels.getInstance();
        List<RobotIdName> robotNames = new ArrayList<>();
        synchronized(rm) {
            List<RobotModel> robots = rm.getRobots();
            for(RobotModel robot : robots) {
                robotNames.add(new RobotIdName(robot));
            }
        }
        return gson.toJson(robotNames);
    }

    @POST
    public String newRobot(String input) {
        try {
            RobotName request = gson.fromJson(input, RobotName.class);
            RobotModels rm = RobotModels.getInstance();
            
            RobotIdName robotName = null;
            synchronized(rm) {
                RobotModel robot = rm.newRobot(request.getName());
                robotName = new RobotIdName(robot);
            }
            return gson.toJson(robotName);
        } catch (JsonSyntaxException e) {
            return "{ \"error\": \"Invalid input. Not a valid json\", \"exception\": \"" + e.toString() + "\"}";
        }
        
    }

}
