package cz.cuni.mff.nprg013.tests.robots.model;

import java.util.Arrays;
import java.util.List;

public class Maze {

    public static final int HEIGHT = 12;
    public static final int WIDTH = 16;
    
    private MazeFields[] maze;
    
    public Maze() {
        this.maze = new MazeFields[HEIGHT*WIDTH];
        for(int row = 0; row < HEIGHT; row++) {
            for(int column = 0; column < WIDTH; column++) {
                setMaze(row,column, MazeFields.EMPTY);
            }
        }
        
        horizontalWall(0, 0, WIDTH);
        horizontalWall(HEIGHT - 1, 0, WIDTH);
        verticalWall(0, 0, HEIGHT);
        verticalWall(0, WIDTH - 1, HEIGHT);
        
        horizontalWall(2, 10, 3);
        horizontalWall(2, 4, 5);
        horizontalWall(2, 10, 3);
        verticalWall(3, 6, 4);
        verticalWall(7, 9, 4);
        
        setMaze(0, 8, MazeFields.EXIT);
    }
    
    
    protected void setMaze(int row, int column, MazeFields field) {
        this.maze[row*WIDTH+column] = field;
    }

    public MazeFields getMaze(int row, int column) {
        return this.maze[row*WIDTH+column];
    }
    protected void horizontalWall(int row, int column, int blocks) {
        for(int i = 0; i < blocks; i++) {
            setMaze(row, column + i, MazeFields.WALL);
        }
    }

    protected void verticalWall(int row, int column, int blocks) {
        for(int i = 0; i < blocks; i++) {
            setMaze(row+i, column, MazeFields.WALL);
        }
    }
    
    public String toString(List<RobotModel> robots) {
        StringBuilder sb = new StringBuilder(maze.length + HEIGHT);
        for(int row = 0; row < HEIGHT; row++) {
            for(int column = 0; column < WIDTH; column++) {
                MazeFields mf = getMaze(row,column);
                if (MazeFields.ROBOT == mf) {
                    RobotModel r = RobotModel.findRobotAtPosition(column, row, robots);
                    sb.append(r.getName().charAt(0));
                } else {
                    sb.append(mf.mazeMark);
                }
            }
            sb.append('\n');
        }
        return sb.toString();
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(maze.length + HEIGHT);
        for(int row = 0; row < HEIGHT; row++) {
            for(int column = 0; column < WIDTH; column++) {
                sb.append(getMaze(row,column).mazeMark);
            }
            sb.append('\n');
        }
        return sb.toString();
    }
    
    public Maze addRobots(List<RobotModel> robots) {
        Maze result = new Maze();
        result.maze = Arrays.copyOf(maze, maze.length);
        
        for(RobotModel robot : robots) {
            if (robot.getState() == RobotState.COMPLETED) {
                continue; // Skipping completed robots at exit node
            }
            if (MazeFields.EMPTY == result.getMaze(robot.getPosY(), robot.getPosX())) {
                result.setMaze(robot.getPosY(), robot.getPosX(), MazeFields.ROBOT);
            }
        }
        return result;
    }
    
    public void setFire(int row, int column) {
        if (getMaze(row,column) == MazeFields.EMPTY) {
            setMaze(row,column, MazeFields.FIRE);
        }
    }
    
    public static void main(String[] args) {
        Maze m = new Maze();
        System.out.println(m);
    }

}
