package cz.cuni.mff.nprg013.tests.robots.model;

import java.util.Iterator;
import java.util.List;

public class Scheduler {
    
    public static final int TURN_SLEEP = 3000;
    
    private Maze mazeWithFires;
    
    public void runSimulation() {
        // Terminates  on InterruptedException
        while(true) {
            RobotModels robotModels = RobotModels.getInstance();
            // Global lock
            synchronized(robotModels) {
                List<RobotModel> robots = robotModels.getRobots();
                Maze maze = new Maze();
                mazeWithFires = executeTurn(maze, robots);
                System.out.println(mazeWithFires.toString(robots));
                System.out.println(mazeWithFires.toString(robots));
                for(RobotModel robot : robots) {
                    System.out.println("\t" + robot);
                }
            }
            System.out.println();
            
            try {
                Thread.sleep(TURN_SLEEP);
            } catch (InterruptedException e) {
                System.out.println("Scheduler is terminating");
                break;
            }
            
        }
    }
    
    
    public Maze getMazeWithFires() {
        return mazeWithFires;
    }


    public Maze executeTurn(Maze m, List<RobotModel> robots) {
        // Step 4 check if timed-out
        for(RobotModel robot : robots) {
            robot.checkTimeout();
        }
        
        // Deleting timed-out robots
        Iterator<RobotModel> it = robots.iterator();
        while (it.hasNext()) {
            RobotModel robot = it.next();
            if (robot.getState() == RobotState.TIMED_OUT) {
                System.out.println(" Removing robot: " + robot.getName() + " (id=" + robot.getId() + ") - no activity ... timeout");
                it.remove();
            }
        }
        
        // Step 1, turn and movements
        for(RobotModel robot : robots) {
            robot.performActionIfMovement(m, robots);
        }
        
        // Step 2, firing
        for(RobotModel robot : robots) {
            robot.performActionIfFire(m, robots);
        }
        
        // Step 3, check if on completed
        for(RobotModel robot : robots) {
            robot.checkExit(m);
        }
        
        for(RobotModel robot : robots) {
            robot.turnCompleted();
        }
       
       
        return m.addRobots(robots);
    }

}
